# Copyright (C) 2013 Intra2net AG
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see
# <http://www.gnu.org/licenses/lgpl-3.0.html>


import os

from deltatar.tarfile import TarFile, GNU_FORMAT

import filesplit
from . import BaseTest
from . import new_volume_handler


class EncryptionTest(BaseTest):
    """
    Test encryption after compression in tarfiles
    """

    def test_openssl_decrypt(self):
        """
        Create a tar file with only one file inside, using concat
        compression and encryption mode. Then decrypt with openssl,
        decompress it with zcat and untar it with gnu tar.
        """

        # create the content of the file to compress and hash it
        hash = self.create_file("big", 50000)

        # create the tar file with volumes
        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="w#gz.aes128",
                              format=GNU_FORMAT,
                              concat_compression=True,
                              password='key')
        tarobj.add("big")
        tarobj.close()
        os.unlink("big")

        # extract with normal tar and check output
        filesplit.split_file(b'Salted__', "sample.tar.gz.aes.", "sample.tar.gz.aes128")

        assert os.path.exists("sample.tar.gz.aes.0") # beginning of the tar file
        assert os.path.exists("sample.tar.gz.aes.1") # first file

        os.system("openssl aes-128-cbc -nopad -k 'key' -d -in sample.tar.gz.aes.1 -out sample.tar.gz")
        os.system("zcat sample.tar.gz 2>/dev/null > sample.tar")
        os.system("tar xf sample.tar")
        assert os.path.exists("big")
        assert hash == self.md5sum("big")

    def test_openssl_multiple_files_decrypt(self):
        """
        Create a tar file with multiple files inside, using concat
        compression and encryption mode. Then decrypt with openssl,
        decompress it with zcat and untar it with gnu tar.
        """

        # create sample data
        hash = dict()
        hash["big"] = self.create_file("big", 50000)
        hash["small"] = self.create_file("small", 100)
        hash["small2"] = self.create_file("small2", 354)

        # create the tar file with volumes
        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="w#gz.aes128",
                              format=GNU_FORMAT,
                              concat_compression=True,
                              password='key')

        for k in hash:
            tarobj.add(k)
        tarobj.close()

        for k in hash:
            os.unlink(k)

        # extract with normal tar and check output
        filesplit.split_file(b'Salted__', "sample.tar.gz.aes.", "sample.tar.gz.aes128")

        assert os.path.exists("sample.tar.gz.aes.0") # beginning of the tar file
        assert os.path.exists("sample.tar.gz.aes.1") # first file
        assert os.path.exists("sample.tar.gz.aes.2") # second file
        assert os.path.exists("sample.tar.gz.aes.3") # third file
        assert not os.path.exists("sample.tar.gz.aes.4") # nothing else

        # extract and check output
        for i in range(1, 4):
            fname = "sample.tar.gz.aes.%d" % i
            os.system("openssl aes-128-cbc -nopad -k 'key' -d -in %s -out sample.tar.gz" % fname)
            os.system("zcat sample.tar.gz 2>/dev/null > sample.tar")
            os.system("tar xf sample.tar")

        for key, value in hash.items():
            assert os.path.exists(key)
            assert value == self.md5sum(key)

    def test_decrypt(self):
        """
        Create a tar file with only one file inside, using concat
        compression and encryption mode. Then decrypt it.
        """

        # create the content of the file to compress and hash it
        hash = self.create_file("big", 50000)

        # create the tar file with volumes
        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="w#gz.aes128",
                              format=GNU_FORMAT,
                              concat_compression=True,
                              password='key')
        tarobj.add("big")
        tarobj.close()
        os.unlink("big")

        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="r#gz.aes128",
                              format=GNU_FORMAT,
                              password='key')
        tarobj.extractall()
        tarobj.close()
        assert os.path.exists("big")
        assert hash == self.md5sum("big")


    def test_multiple_file_decrypt(self):
        """
        Create a tar file with only one file inside, using concat
        compression and encryption mode. Then decrypt it.
        """

        # create sample data
        hash = dict()
        hash["big"] = self.create_file("big", 50000)
        hash["small"] = self.create_file("small", 100)
        hash["small2"] = self.create_file("small2", 354)

        # create the tar file with volumes
        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="w#gz.aes128",
                              format=GNU_FORMAT,
                              concat_compression=True,
                              password='key')

        for k in hash:
            tarobj.add(k)
        tarobj.close()

        for k in hash:
            os.unlink(k)

        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="r#gz.aes128",
                              format=GNU_FORMAT,
                              password='key')
        tarobj.extractall()
        tarobj.close()

        for key, value in hash.items():
            assert os.path.exists(key)
            assert value == self.md5sum(key)

    def test_multivol_file_decrypt(self):
        '''
        Test multivol tarball with encryption.
        '''

        # create sample data
        hash = dict()
        hash["big"] = self.create_file("big", 50000)
        hash["big2"] = self.create_file("big2", 10200)
        hash["small"] = self.create_file("small", 100)
        hash["small2"] = self.create_file("small2", 354)

        # create the tar file with volumes
        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="w#gz.aes128",
                              password='key',
                              concat_compression=True,
                              max_volume_size=20000,
                              new_volume_handler=new_volume_handler)

        for k in hash:
            tarobj.add(k)
        tarobj.close()

        assert os.path.exists("sample.tar.gz.aes128")
        for k in hash:
            os.unlink(k)

        # extract
        tarobj = TarFile.open("sample.tar.gz.aes128",
                              mode="r#gz.aes128",
                              password="key",
                              new_volume_handler=new_volume_handler)
        tarobj.extractall()
        tarobj.close()

        # check output
        for key, value in hash.items():
            assert os.path.exists(key)
            assert value == self.md5sum(key)

    def test_multivol_file_decrypt_aes256(self):
        '''
        Test multivol tarball with encryption aes256
        '''

        # create sample data
        hash = dict()
        hash["big"] = self.create_file("big", 50000)
        hash["big2"] = self.create_file("big2", 10200)
        hash["small"] = self.create_file("small", 100)
        hash["small2"] = self.create_file("small2", 354)

        # create the tar file with volumes
        tarobj = TarFile.open("sample.tar.gz.aes256",
                              mode="w#gz.aes256",
                              password='key',
                              concat_compression=True,
                              max_volume_size=20000,
                              new_volume_handler=new_volume_handler)

        for k in hash:
            tarobj.add(k)
        tarobj.close()

        assert os.path.exists("sample.tar.gz.aes256")
        for k in hash:
            os.unlink(k)

        # extract
        tarobj = TarFile.open("sample.tar.gz.aes256",
                              mode="r#gz.aes256",
                              password="key",
                              new_volume_handler=new_volume_handler)
        tarobj.extractall()
        tarobj.close()

        # check output
        for key, value in hash.items():
            assert os.path.exists(key)
            assert value == self.md5sum(key)

    def test_openssl_decrypt_256(self):
        """
        Create a tar file with only one file inside, using concat
        compression and encryption mode. Then decrypt with openssl,
        decompress it with zcat and untar it with gnu tar.

        Using aes 256.
        """

        # create the content of the file to compress and hash it
        hash = self.create_file("big", 50000)

        # create the tar file with volumes
        tarobj = TarFile.open("sample.tar.gz.aes256",
                              mode="w#gz.aes256",
                              format=GNU_FORMAT,
                              concat_compression=True,
                              password='key')
        tarobj.add("big")
        tarobj.close()
        os.unlink("big")

        # extract with normal tar and check output
        filesplit.split_file(b'Salted__', "sample.tar.gz.aes.", "sample.tar.gz.aes256")

        assert os.path.exists("sample.tar.gz.aes.0") # beginning of the tar file
        assert os.path.exists("sample.tar.gz.aes.1") # first file

        os.system("openssl aes-256-cbc -nopad -k 'key' -d -in sample.tar.gz.aes.1 -out sample.tar.gz")
        os.system("zcat sample.tar.gz 2>/dev/null > sample.tar")
        os.system("tar xf sample.tar")
        assert os.path.exists("big")
        assert hash == self.md5sum("big")
